"""Init for CBS."""

from __future__ import absolute_import
from .cbseg import determine_cbs_stat, determine_t_stat, determine_cbs, segment, validate
from .cbseg import get_include

# This is extracted automatically by the top-level setup.py.
__version__ = '0.1.0'